package com.citygis.template.domain.component.redis;

import com.citygis.template.domain.utils.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.insert.Insert;
import net.sf.jsqlparser.statement.update.Update;
import net.sf.jsqlparser.util.TablesNamesFinder;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.redisson.api.RBucket;
import org.redisson.api.RKeys;
import org.redisson.api.RSet;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import jakarta.annotation.Resource;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Aspect
@Component
@Slf4j
public class RedisProcess<T> {
    @Resource
    private RedissonClient redissonClient;
    // which can directly get the type collection of toString.
    private static final List<String> typeList;

    static {
        ArrayList<String> rs = new ArrayList<>();
        rs.add(String.class.getTypeName());
        rs.add(int.class.getTypeName());
        rs.add(Integer.class.getTypeName());
        rs.add(double.class.getTypeName());
        rs.add(BigDecimal.class.getTypeName());
        rs.add(boolean.class.getTypeName());
        rs.add(byte.class.getTypeName());
        rs.add(Date.class.getTypeName());
        rs.add(Long.class.getTypeName());
        typeList = rs;
    }

    /**
     * method to intercept all meta annotations redisCache annotations
     */
    @Pointcut("@annotation(com.citygis.template.domain.component.redis.RedisCache)")
    public void redisCachePoint() {
    }

    @Value("${redis.cache.enabled}")
    private String redisCacheEnable;

    /**
     * @param joinPoint join point
     * @return result of method executed
     */
    @Around("redisCachePoint()")
    public T around(ProceedingJoinPoint joinPoint) {
        if (redisCacheEnable.equals("true")) {
            long begin = System.currentTimeMillis();
            RedisCache cacheCfg = getCacheConfig(joinPoint);
            assert cacheCfg != null;
            String tableName = cacheCfg.key().split(":")[0];
            String key = getKeyFromAnnotation(joinPoint, cacheCfg);
            StringBuilder stringBuilder = new StringBuilder("key is: " + key + ". ");
            RBucket<T> bucket = redissonClient.getBucket(key);
            T redisValue = bucket.get();
            if (null != redisValue) {
                long end = System.currentTimeMillis();
                stringBuilder.append("cache time is: ").append(end - begin).append(" ms");
                info(stringBuilder.toString());
                return (T) redisValue;
            }
            try {
                T result = (T) joinPoint.proceed();
                if (result != null && filterEmpty(result)) {
                    if (!tableName.equalsIgnoreCase("api")) {
                        String setKey = "Resource" + ":" + tableName;
                        RSet<String> keySet = redissonClient.getSet(setKey);
                        keySet.expire(cacheCfg.expire(), TimeUnit.MILLISECONDS);
                        keySet.add(key);
                        stringBuilder.append("keySet is ").append(setKey).append(". ");
                    }
                    bucket.set(result, cacheCfg.expire(), TimeUnit.MILLISECONDS);
                    long end = System.currentTimeMillis();
                    stringBuilder.append("DB time is: ").append(end - begin).append(" ms");
                    info(stringBuilder.toString());
                }
                return result;
            } catch (Throwable e) {
                error(e.getMessage(), e);
            }
        } else {
            try {
                return (T) joinPoint.proceed();
            } catch (Throwable e) {
                error(e.getMessage(), e);
            }
        }
        return null;
    }

    public void interceptor(String sql) {
        if (redisCacheEnable.equals("true")) {
            try {
                String structSql = sql.replaceAll("\n", "");
                Statement stmt = CCJSqlParserUtil.parse(structSql);
                if (stmt instanceof Update || stmt instanceof Insert) {
                    TablesNamesFinder tablesNamesFinder = new TablesNamesFinder();
                    List<String> tableName = tablesNamesFinder.getTableList(stmt);
                    log.info("Intercepted SQL:" + structSql.replaceAll("  ", "") + "; Table name is: " + tableName);
                    String setKey = "Resource" + ":" + tableName.get(0);
                    RSet<String> keysSet = redissonClient.getSet(setKey);
                    if (keysSet != null && keysSet.size() > 0) {
                        List<String> deleteKeys = new ArrayList<>(keysSet);
                        deleteKeys.add(setKey);
                        RKeys rKeys = redissonClient.getKeys();
                        rKeys.delete(deleteKeys.toArray(new String[]{}));
                        log.info("Delete keys: " + JsonUtil.toJsonString(deleteKeys));
                    } else {
                        log.info("This table has no associated key!");
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    private RedisCache getCacheConfig(ProceedingJoinPoint joinPoint) {
        String methodName = joinPoint.getSignature().getName();
        RedisCache result;
        Method[] methods = joinPoint.getTarget().getClass().getMethods();
        for (Method m : methods) {
            if (methodName.equals(m.getName())) {
                Annotation[] annotations = m.getDeclaredAnnotations();
                for (Annotation annotation : annotations) {
                    if (annotation instanceof RedisCache) {
                        result = (RedisCache) annotation;
                        return result;
                    }
                }
            }
        }
        return null;
    }

    private String getKeyFromAnnotation(ProceedingJoinPoint joinPoint, RedisCache cacheCfg) {
        Object[] args = joinPoint.getArgs();
        StringBuilder key = new StringBuilder("Resource:" + cacheCfg.key());
        key.append("@");
        for (Object o : args) {
            String oType = o.getClass().getTypeName();
            if (typeList.contains(oType)) {
                key.append(o).append(',');
            } else {
                key.append(JsonUtil.toJsonString(o)).append(',');
            }
        }
        key.deleteCharAt(key.length() - 1);
        return key.toString();
    }

    private boolean filterEmpty(T input) {
        if (input instanceof List) {
            return !((List<?>) input).isEmpty();
        }
        if (input instanceof Map) {
            return !((Map<?, ?>) input).isEmpty();
        }
        return true;
    }

    private void info(String message) {
        log.info(message);
    }

    private void error(String message, Throwable eThrowable) {
        log.error(message, eThrowable);
    }
}
